package com.starbuds.server.im.server;

import com.starbuds.server.im.server.rocket.RocketConsumer;
import com.starbuds.server.im.server.ws.WsStarter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;

import java.net.InetAddress;
import java.net.UnknownHostException;

/**
 * SpringBoot 入口
 *
 * @author Lay
 * @date 2020/1/5
 */
@SpringBootApplication
@Slf4j
public class Application implements CommandLineRunner {
    public static void main(String[] args) {
        System.setProperty("app.name", "im-server");
        try {
            System.setProperty("hostname", InetAddress.getLocalHost().getHostName());
        } catch (UnknownHostException e) {
            System.setProperty("hostname", System.getProperty("app.name"));
        }
        SpringApplication.run(Application.class, args);
        log.info("[im-server] started.");
    }

    @Autowired
    RocketConsumer rocketConsumer;

    @Autowired
    WsStarter wsStarter;

    @Override
    public void run(String... args) throws Exception {
        // 启动 Websocket
        wsStarter.start();

        // 启动 RocketMQ 监听
        rocketConsumer.start();
    }
}
