package com.starbuds.server.im.server.rocket;

import com.aliyun.openservices.ons.api.Consumer;
import com.aliyun.openservices.ons.api.ONSFactory;
import com.aliyun.openservices.ons.api.PropertyKeyConst;
import com.aliyun.openservices.ons.api.PropertyValueConst;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.im.server.rocket.listener.LiveMsgListener;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.DisposableBean;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.List;
import java.util.Properties;

/**
 * Class
 *
 * @author Lay
 * @date 2019/05/30
 */
@Component
@Slf4j
public class RocketConsumer implements DisposableBean {
    @Resource
    RocketConfig rocketConfig;

    @Resource
    LiveMsgListener liveMsgListener;

    private Consumer consumer;

    @Override
    public void destroy() throws Exception {
        if (consumer != null) {
            consumer.shutdown();
        }
    }

    public void start() {
        log.info("RocketMQ 消费者初始化");
        Properties properties = rocketConfig.getProps();

        // 广播模式订阅
        properties.put(PropertyKeyConst.MessageModel, PropertyValueConst.BROADCASTING);
        consumer = ONSFactory.createConsumer(properties);

        log.info("RocketMQ 消费者启动rocketConfig.getProps();Properties=" + properties.toString());
        consumer.subscribe(RocketTopic.TPC_LIVE, getSubscribeTags(Arrays.asList(
                RocketTag.LIVE_ROOM,
                RocketTag.CHAT_GROUP,
                RocketTag.RTC_ROOM,
                RocketTag.LIVE_TEXT
        )), liveMsgListener);
        consumer.start();
    }

    private String getSubscribeTags(List<String> tags) {
        int size = tags.size();
        StringBuilder sbTags = new StringBuilder();
        for (int i = 0; i < size; i++) {
            sbTags.append(tags.get(i));
            if (i != size - 1) {
                sbTags.append("||");
            }
        }
        return sbTags.toString();
    }
}
