package com.starbuds.server.im.server.rocket.handler;

import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.pojo.mq.QueueChatGroupMsg;
import com.starbuds.server.common.pojo.mq.QueueLiveRoomMsg;
import com.starbuds.server.common.pojo.mq.QueueRtcRoomMsg;
import com.starbuds.server.common.service.user.RtcUserService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.im.server.constant.GroupChannel;
import com.starbuds.server.im.server.rocket.entity.QueueLiveTextMsg;
import com.starbuds.server.im.server.util.MsgUtil;
import com.starbuds.server.im.server.ws.WsStarter;
import com.starbuds.server.im.server.ws.entity.MsgPacket;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;
import org.tio.core.Tio;

import javax.annotation.Resource;

/**
 * 直播消息处理
 *
 * @author Lay
 * @date 2020/1/6
 */
@Component
@Slf4j
public class LiveMsgHandler {

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    RtcUserService rtcUserService;

    @Resource
    WsStarter wsStarter;

    /**
     * 投递直播间消息处理
     *
     * @param liveRoomMsg
     */
    public void handleLiveRoomMsg(QueueLiveRoomMsg liveRoomMsg) {
        MsgPacket msgPacket = new MsgPacket();
        msgPacket.setMsg(JSON.parseObject(liveRoomMsg.getMsgData()));
        msgPacket.setSenderId(liveRoomMsg.getSenderId());
        msgPacket.setTargetId(liveRoomMsg.getTargetId());
        msgPacket.setTargetType(liveRoomMsg.getTargetType());
        // 组装用户信息
        if (liveRoomMsg.getSenderId() != null && liveRoomMsg.getTargetId() != null) {
            msgPacket.setSenderProfile(userService.getLiveUserProfile(liveRoomMsg.getSenderId(), liveRoomMsg.getTargetId()));
        }
        log.info("发送消息 {}", JSON.toJSONString(msgPacket));
        if (ImTargetType.USER == liveRoomMsg.getTargetType()) {
            // 转发用户
            Tio.sendToUser(wsStarter.getServerTioConfig(), String.valueOf(liveRoomMsg.getTargetId()), MsgUtil.buildMsg(msgPacket));
        } else if (ImTargetType.GROUP == liveRoomMsg.getTargetType()) {
            // 转发直播间
            Tio.sendToGroup(wsStarter.getServerTioConfig(), String.valueOf(liveRoomMsg.getTargetId()), MsgUtil.buildMsg(msgPacket));
        } else if (ImTargetType.ALL == liveRoomMsg.getTargetType()) {
            // 全频道通知
            Tio.sendToGroup(wsStarter.getServerTioConfig(), GroupChannel.CHANNEL_LIVE, MsgUtil.buildMsg(msgPacket));
        }
    }

    /**
     * 投递聊天群组消息
     *
     * @param chatGroupMsg
     */
    public void handleChatGroupMsg(QueueChatGroupMsg chatGroupMsg) {
        MsgPacket msgPacket = new MsgPacket();
        msgPacket.setMsg(JSON.parseObject(chatGroupMsg.getMsgData()));
        msgPacket.setGroupId(chatGroupMsg.getGroupId());
        msgPacket.setSenderId(chatGroupMsg.getSenderId());
        msgPacket.setTargetId(chatGroupMsg.getTargetId());
        msgPacket.setTargetType(chatGroupMsg.getTargetType());
        // 组装用户信息
        if (chatGroupMsg.getSenderId() > 0 && StringUtils.isNotBlank(chatGroupMsg.getGroupId())) {
            msgPacket.setSenderProfile(userService.getChatUserProfile(chatGroupMsg.getSenderId(), chatGroupMsg.getGroupId()));
        }
        log.info("发送消息 {}", JSON.toJSONString(msgPacket));
        if (ImTargetType.USER == chatGroupMsg.getTargetType()) {
            // 转发用户
            Tio.sendToUser(wsStarter.getServerTioConfig(), String.valueOf(chatGroupMsg.getTargetId()), MsgUtil.buildMsg(msgPacket));
        } else if (ImTargetType.GROUP == chatGroupMsg.getTargetType()) {
            // 转发直播间
            Tio.sendToGroup(wsStarter.getServerTioConfig(), String.valueOf(chatGroupMsg.getGroupId()), MsgUtil.buildMsg(msgPacket));
        } else if (ImTargetType.ALL == chatGroupMsg.getTargetType()) {
            // 全频道通知
            Tio.sendToGroup(wsStarter.getServerTioConfig(), GroupChannel.CHANNEL_GROUP, MsgUtil.buildMsg(msgPacket));
        }
    }

    public void handleRtcRoomMsg(QueueRtcRoomMsg rtcRoomMsg) {
        MsgPacket msgPacket = new MsgPacket();
        msgPacket.setMsg(JSON.parseObject(rtcRoomMsg.getMsgData()));
        msgPacket.setGroupId(String.valueOf(rtcRoomMsg.getRoomId()));
        msgPacket.setSenderId(rtcRoomMsg.getSenderId());
        msgPacket.setTargetId(rtcRoomMsg.getTargetId());
        msgPacket.setTargetType(rtcRoomMsg.getTargetType());
        // 组装用户信息
        if (rtcRoomMsg.getSenderId() > 0 && Convert.toLong(rtcRoomMsg.getRoomId(), 0L) > 0) {
            msgPacket.setSenderProfile(rtcUserService.getUserProfile(rtcRoomMsg.getSenderId(), rtcRoomMsg.getRoomId()));
        }
        log.info("发送消息 {}", JSON.toJSONString(msgPacket));
        if (ImTargetType.USER == rtcRoomMsg.getTargetType()) {
            // 转发用户
            Tio.sendToUser(wsStarter.getServerTioConfig(), String.valueOf(rtcRoomMsg.getTargetId()), MsgUtil.buildMsg(msgPacket));
        } else if (ImTargetType.GROUP == rtcRoomMsg.getTargetType()) {
            // 转发直播间
            Tio.sendToGroup(wsStarter.getServerTioConfig(), String.valueOf(rtcRoomMsg.getRoomId()), MsgUtil.buildMsg(msgPacket));
        } else if (ImTargetType.ALL == rtcRoomMsg.getTargetType()) {
            // 全频道通知
            Tio.sendToGroup(wsStarter.getServerTioConfig(), GroupChannel.CHANNEL_RTC, MsgUtil.buildMsg(msgPacket));
        }
    }

    /**
     * 发送消息
     *
     * @param liveTextMsg
     */
    public void handleLiveTextMsg(QueueLiveTextMsg liveTextMsg) {
        Tio.sendToGroup(wsStarter.getServerTioConfig(), liveTextMsg.getGroupId(), MsgUtil.buildMsg(liveTextMsg.getMsgPacket()));
    }
}
