package com.starbuds.server.im.server.rocket.listener;

import com.alibaba.fastjson.JSON;
import com.aliyun.openservices.ons.api.Action;
import com.aliyun.openservices.ons.api.ConsumeContext;
import com.aliyun.openservices.ons.api.Message;
import com.aliyun.openservices.ons.api.MessageListener;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.pojo.mq.QueueChatGroupMsg;
import com.starbuds.server.common.pojo.mq.QueueLiveRoomMsg;
import com.starbuds.server.common.pojo.mq.QueueRtcRoomMsg;
import com.starbuds.server.im.server.rocket.entity.QueueLiveTextMsg;
import com.starbuds.server.im.server.rocket.handler.LiveMsgHandler;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * 备注
 *
 * @author Lay
 * @date 2020/1/6
 */
@Component
@Slf4j
public class LiveMsgListener implements MessageListener {

    @Resource
    LiveMsgHandler liveMsgHandler;

    @Override
    public Action consume(Message message, ConsumeContext consumeContext) {
        try {
            String tag = message.getTag();
            switch (tag) {
                case RocketTag.LIVE_ROOM:
                    liveMsgHandler.handleLiveRoomMsg(JSON.parseObject(message.getBody(), QueueLiveRoomMsg.class));
                    break;
                case RocketTag.CHAT_GROUP:
                    liveMsgHandler.handleChatGroupMsg(JSON.parseObject(message.getBody(), QueueChatGroupMsg.class));
                    break;
                case RocketTag.RTC_ROOM:
                    liveMsgHandler.handleRtcRoomMsg(JSON.parseObject(message.getBody(), QueueRtcRoomMsg.class));
                    break;
                case RocketTag.LIVE_TEXT:
                    liveMsgHandler.handleLiveTextMsg(JSON.parseObject(message.getBody(), QueueLiveTextMsg.class));
                    break;
                default:
            }
        } catch (Exception e) {
            log.error("消息处理失败", e);
            return Action.ReconsumeLater;
        }
        return Action.CommitMessage;
    }
}