package com.starbuds.server.im.server.ws;

import cn.hutool.core.convert.Convert;
import com.starbuds.server.im.server.ws.config.ImServerConfig;
import com.starbuds.server.im.server.ws.config.PropertyKeyConst;
import com.starbuds.server.im.server.ws.handler.WsMsgHandler;
import com.starbuds.server.im.server.ws.listener.ImGroupListener;
import com.starbuds.server.im.server.ws.listener.ImIpStatListener;
import com.starbuds.server.im.server.ws.listener.ImServerAioListener;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.tio.server.ServerTioConfig;
import org.tio.websocket.server.WsServerConfig;
import org.tio.websocket.server.WsServerStarter;

/**
 * Websocket 启动器
 *
 * @author Lay
 * @date 2020/1/5
 */
@Component
@Slf4j
public class WsStarter {
    /**
     * websocket自身服务启动器
     */
    private WsServerStarter wsServerStarter;

    /**
     * websocket自身服务配置
     */
    private ServerTioConfig serverTioConfig;

    /**
     * 业务参数设置如主机、端口等
     */
    @Autowired
    ImServerConfig imServerConfig;

    /**
     * 消息处理
     */
    @Autowired
    WsMsgHandler wsMsgHandler;


    /**
     *
     */
    @Autowired
    ImGroupListener imGroupListener;

    /**
     *
     */
    @Autowired
    ImServerAioListener imServerAioListener;

    /**
     * IP 状态监听
     */
    @Autowired
    ImIpStatListener imIpStatListener;


    /**
     * 初始化
     *
     * @return
     */
    private boolean init() {
        WsServerConfig config = new WsServerConfig(Convert.toInt(imServerConfig.getProps().get(PropertyKeyConst.PORT)));
        config.setBindIp(imServerConfig.getProps().getProperty(PropertyKeyConst.HOST));

        try {
            wsServerStarter = new WsServerStarter(config, wsMsgHandler);
            // 关闭自带的新版本检查
            wsServerStarter.getTioServer().setCheckLastVersion(false);
            log.info("Websocket 启动器初始化成功");
            return true;

        } catch (Exception e) {
            log.error("Websocket 启动器初始化失败", e);
        }

        return false;
    }

    /**
     * 初始化上下文监听器
     */
    private void initServerConfig() {
        serverTioConfig = wsServerStarter.getServerTioConfig();
        // 协议名称
        serverTioConfig.setName(ImServerConfig.PROTOCOL_NAME);

        // 群组事件监听
        serverTioConfig.setGroupListener(imGroupListener);

        // 事件监听
        serverTioConfig.setServerAioListener(imServerAioListener);

        // IP 监控
        serverTioConfig.setIpStatListener(imIpStatListener);

        // 设置 IP 统计时段
        serverTioConfig.ipStats.addDurations(ImServerConfig.IPSTAT_DURATIONS);

        // 取消框架心跳检测
        serverTioConfig.setHeartbeatTimeout(Convert.toLong(imServerConfig.getProps().get(PropertyKeyConst.HEARTBEAT_TIMEOUT)));
    }

    public ServerTioConfig getServerTioConfig() {
        return serverTioConfig;
    }

    public void start() {
        if (init()) {
            // 初始化上下文相关
            initServerConfig();

            // 启动服务
            try {
                wsServerStarter.start();
                log.info("websocket 服务已启动");
            } catch (Exception e) {
                log.info("websocket 服务启动失败");
            }
        }
    }
}
