package com.starbuds.server.im.server.ws.config;

import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;
import org.tio.utils.time.Time;

import java.util.Properties;

/**
 * IM 服务配置
 *
 * @author Lay
 * @date 2020/1/5
 */
@Configuration
@Slf4j
public class ImServerConfig {
    /**
     * 协议名称
     */
    public static final String PROTOCOL_NAME = "starbuds";

    /**
     * IP 监听时间段
     */
    public static final Long[] IPSTAT_DURATIONS = {
            Time.MINUTE_1 * 5
    };

    @Value("${websocket.host}")
    String host;

    @Value("${websocket.port}")
    int port;

    @Value("${websocket.heartbeat.timeout}")
    long heartbeatTimeout;

    @Value("${websocket.heartbeat.ping}")
    String heartbeatPing;

    @Value("${websocket.heartbeat.pong}")
    String heartbeatPong;

    /**
     * 配置
     */
    private Properties properties;

    public Properties getProps() {
        if (properties == null) {
            properties = new Properties();
            properties.put(PropertyKeyConst.HOST, host);
            properties.put(PropertyKeyConst.PORT, port);
            properties.put(PropertyKeyConst.HEARTBEAT_TIMEOUT, heartbeatTimeout);
            properties.put(PropertyKeyConst.HEARTBEAT_PING, heartbeatPing);
            properties.put(PropertyKeyConst.HEARTBEAT_PONG, heartbeatPong);
        }

        return properties;
    }
}
