package com.starbuds.server.im.server.ws.handler;

import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.ClientType;
import com.starbuds.server.common.constant.data.WordFilterTypeEnum;
import com.starbuds.server.common.constant.message.im.ImMsgType;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.db.PrivateMsgDO;
import com.starbuds.server.common.pojo.user.TokenBO;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.data.WordFilterService;
import com.starbuds.server.common.service.media.SingleChatService;
import com.starbuds.server.common.service.media.VoiceChatService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.rtc.RtcRoomService;
import com.starbuds.server.common.service.user.AnchorService;
import com.starbuds.server.common.service.user.RtcUserService;
import com.starbuds.server.common.service.user.SecurityService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.common.constant.cache.RedisUserKeys;
import com.starbuds.server.im.server.constant.ChatMode;
import com.starbuds.server.im.server.constant.GroupChannel;
import com.starbuds.server.im.server.constant.HandshakeParam;
import com.starbuds.server.im.server.constant.ImConstants;
import com.starbuds.server.im.server.rocket.RocketProducer;
import com.starbuds.server.im.server.rocket.entity.QueueLiveTextMsg;
import com.starbuds.server.im.server.util.GzipUtil;
import com.starbuds.server.im.server.util.MsgUtil;
import com.starbuds.server.im.server.ws.config.ImServerConfig;
import com.starbuds.server.im.server.ws.config.PropertyKeyConst;
import com.starbuds.server.im.server.ws.entity.MsgPacket;
import com.sun.xml.internal.bind.v2.TODO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;
import org.tio.core.ChannelContext;
import org.tio.core.Tio;
import org.tio.http.common.HttpRequest;
import org.tio.http.common.HttpResponse;
import org.tio.websocket.common.WsRequest;
import org.tio.websocket.common.WsSessionContext;
import org.tio.websocket.server.handler.IWsMsgHandler;

import javax.annotation.Resource;
import java.util.List;
import java.util.Optional;
import java.util.Properties;

/**
 * 消息处理
 *
 * @author Lay
 * @date 2020/1/5
 */
@Component
@Slf4j
public class WsMsgHandler implements IWsMsgHandler {

    @Reference(version = "1.0.0")
    SecurityService securityService;

    @Reference(version = "1.0.0")
    AnchorService anchorService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    WordFilterService wordFilterService;

    @Reference(version = "1.0.0")
    SingleChatService singleChatService;

    @Reference(version = "1.0.0")
    VoiceChatService voiceChatService;

    @Reference(version = "1.0.0")
    RtcRoomService rtcRoomService;

    @Reference(version = "1.0.0")
    RtcUserService rtcUserService;

    @Resource
    ImServerConfig imServerConfig;

    @Resource
    RocketProducer rocketProducer;

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Reference(version = "1.0.0")
    ImService imService;

    /**
     * 游客 Token
     */
    public static final String GUEST_TOKEN = "GUEST";

    /**
     * 握手方法
     *
     * ws://127.0.0.1:2020?token=用户信息中token(获取用户uid)&cid=201&rri=语音房间号
     * ws://127.0.0.1:2020?token=xxx&cid=201&rri=xx，不同url连接im-server会产生不同的tcp连接，channelContext也会不同，
     * 每次进入要创建和关闭WebSocket连接，私聊也是单独url连接，性能不好。后期修改为一个用户只和im-server产生一个，加入群组时发消息绑定群组，退出直播间时解绑。
     * @param httpRequest
     * @param httpResponse
     * @param channelContext
     * @return
     * @throws Exception
     */
    @Override
    public HttpResponse handshake(HttpRequest httpRequest, HttpResponse httpResponse, ChannelContext channelContext) throws Exception {
        log.info("开始握手了哦====");
        // 通过 token 校验用户
        String token = httpRequest.getParam(HandshakeParam.TOKEN);
        // 渠道号
        Long channelId = Convert.toLong(httpRequest.getParam(HandshakeParam.CHANNEL_ID));

        // 秀场直播间（主播开播，房间号roomid就是主播用户id）rid=%@
        String rid = httpRequest.getParam(HandshakeParam.ROOM_ID);

        // 语音房间rri=%@
        String rri = httpRequest.getParam(HandshakeParam.RTC_ROOM_ID);

        // 单聊视频通话，如业务叫频道，其实就和群组一样如头条
        String cn = httpRequest.getParam(HandshakeParam.CHANNEL_NAME);

        // 前端没token连接时要赋值token=GUEST，实现平台系统消息的接收（客户端暂时都在登录成功后有token才连接im-server）
        if (StringUtils.isNotBlank(token)) {

            // 秀场直播间（主播开播，房间号roomid就是主播用户id）rid=%@
            if (StringUtils.isNotBlank(rid)) {
                if (anchorService.isAnchor(Convert.toLong(rid))) {
                    // 直播模式
                    channelContext.set(ImConstants.MODE, ChatMode.LIVE);
                    if (GUEST_TOKEN.equals(token)) {
                        // 游客模式
                        channelContext.set(ImConstants.GUEST, BoolIntVal.True);
                        // 绑定渠道
                        channelContext.set("channelId", channelId);
                        return httpResponse;
                    } else {
                        TokenBO tokenBO = securityService.getToken(token);
                        if (tokenBO != null) {
                            // 非游客模式
                            channelContext.set(ImConstants.GUEST, BoolIntVal.False);
                            // 绑定 id
                            Tio.bindUser(channelContext, String.valueOf(tokenBO.getUserId()));
                            // 绑定渠道
                            channelContext.set("channelId", channelId);
                            channelContext.set("userId", tokenBO.getUserId());
                            // APP 和 WEB 互斥
                            if (ClientType.iOS == tokenBO.getClientType() || ClientType.Android == tokenBO.getClientType()) {
                                Tio.bindBsId(channelContext, String.format("APP_%d", tokenBO.getUserId()));
                            } else {
                                Tio.bindBsId(channelContext, String.format("%s_%d", tokenBO.getClientType().getName(), tokenBO.getUserId()));
                            }
                            return httpResponse;
                        }
                    }
                }

                // 单聊视频通话cn=%@
            } else if (StringUtils.isNotBlank(cn)) {
                TokenBO tokenBO = securityService.getToken(token);
                if (tokenBO != null) {
                    if (singleChatService.hasConnectPermission(cn, tokenBO.getUserId()) || voiceChatService.hasConnectPermission(cn, tokenBO.getUserId())) {
                        // 单聊模式
                        channelContext.set(ImConstants.MODE, ChatMode.SINGLE_CHAT);
                        // 绑定 id
                        Tio.bindUser(channelContext, String.valueOf(tokenBO.getUserId()));
                        // 绑定渠道
                        channelContext.set("channelId", channelId);
                        channelContext.set("userId", tokenBO.getUserId());
                        // APP 和 WEB 互斥
                        if (ClientType.iOS == tokenBO.getClientType() || ClientType.Android == tokenBO.getClientType()) {
                            Tio.bindBsId(channelContext, String.format("APP_%d", tokenBO.getUserId()));
                        } else {
                            Tio.bindBsId(channelContext, String.format("%s_%d", tokenBO.getClientType().getName(), tokenBO.getUserId()));
                        }
                    } else {
                        Tio.close(channelContext, "没有加入权限");
                    }
                    return httpResponse;
                }

                // 语音房间rri=%@
            } else if (StringUtils.isNotBlank(rri)) {
                if (rtcRoomService.isRoomExists(Convert.toLong(rri))) {
                    // 直播模式
                    channelContext.set(ImConstants.MODE, ChatMode.RTC);
                    if (GUEST_TOKEN.equals(token)) {
                        // 游客模式
                        channelContext.set(ImConstants.GUEST, BoolIntVal.True);
                        // 绑定渠道
                        channelContext.set("channelId", channelId);
                        return httpResponse;
                    } else {
                        TokenBO tokenBO = securityService.getToken(token);
                        if (tokenBO != null) {
                            // 非游客模式
                            channelContext.set(ImConstants.GUEST, BoolIntVal.False);
                            // 绑定渠道
                            channelContext.set("channelId", channelId);
                            channelContext.set("userId", tokenBO.getUserId());
                            // 绑定 id
                            Tio.bindUser(channelContext, String.valueOf(tokenBO.getUserId()));
                            // APP 和 WEB 互斥
                            if (ClientType.iOS == tokenBO.getClientType() || ClientType.Android == tokenBO.getClientType()) {
                                Tio.bindBsId(channelContext, String.format("APP_%d", tokenBO.getUserId()));
                            } else {
                                Tio.bindBsId(channelContext, String.format("%s_%d", tokenBO.getClientType().getName(), tokenBO.getUserId()));
                            }
                            return httpResponse;
                        }
                    }
                }

                // 要进入群组相关时，连接时url中加rid、cn、rri这些参数
                // String rid = httpRequest.getParam(HandshakeParam.ROOM_ID);
                // String cn = httpRequest.getParam(HandshakeParam.CHANNEL_NAME);
                // String rri = httpRequest.getParam(HandshakeParam.RTC_ROOM_ID);
                // rid、cn、rri群聊相关的都没有时且uid有值就是1v1私聊模式
            }else {
                //TokenBO tokenBO = securityService.getToken(token);
                TokenBO tokenBO = new TokenBO();
                //TODO 测试传的token=uid
                tokenBO.setUserId(Convert.toLong(token));
                tokenBO.setClientType(ClientType.iOS);
                if(tokenBO != null){
                    // 私聊模式
                    channelContext.set(ImConstants.MODE, ChatMode.PRIVATE_CHAT);
                    // 非游客模式
                    channelContext.set(ImConstants.GUEST, BoolIntVal.False);
                    // 绑定渠道
                    channelContext.set("channelId", channelId);
                    channelContext.set("userId", tokenBO.getUserId());
                    // 绑定 id
                    Tio.bindUser(channelContext, String.valueOf(tokenBO.getUserId()));
                    // APP 和 WEB 互斥
                    if (ClientType.iOS == tokenBO.getClientType() || ClientType.Android == tokenBO.getClientType()) {
                        Tio.bindBsId(channelContext, String.format("APP_%d", tokenBO.getUserId()));
                    } else {
                        Tio.bindBsId(channelContext, String.format("%s_%d", tokenBO.getClientType().getName(), tokenBO.getUserId()));
                    }
                    return httpResponse;
                }
            }
        }

        // 关闭连接
        Tio.close(channelContext, "认证失败 Token:" + Optional.ofNullable(token).orElse("null"));
        return httpResponse;
    }

    /**
     * 握手后
     *
     * @param httpRequest
     * @param httpResponse
     * @param channelContext
     * @throws Exception
     */
    @Override
    public void onAfterHandshaked(HttpRequest httpRequest, HttpResponse httpResponse, ChannelContext channelContext) throws Exception {
        log.info("握手成功后触发该方法====");
        // 保存用户在线状态
        String token = httpRequest.getParam(HandshakeParam.TOKEN);
        // TODO
        // TokenBO tokenBO = securityService.getToken(token);
        TokenBO tokenBO = new TokenBO();
        // 测试传的token=uid
        String uid = token;
        tokenBO.setUserId(Convert.toLong(uid));
        tokenBO.setClientType(ClientType.iOS);
        redisService.sAdd(RedisUserKeys.USER_ONLINE, uid);

        Integer mode = Convert.toInt(channelContext.get(ImConstants.MODE));
        if (mode != null) {
            // 秀场直播间（主播开播，房间号roomid就是主播用户id）rid=%@
            if (ChatMode.LIVE == mode) {
                // 直播模式
                boolean isGuest = BoolIntVal.True == Convert.toInt(channelContext.get(ImConstants.GUEST), BoolIntVal.True);
                String rid = httpRequest.getParam(HandshakeParam.ROOM_ID);
                // 直播模式
                if (!isGuest) {
                    // 判断是否被拉黑
                    if (anchorService.isBlack(Convert.toLong(rid), Convert.toLong(channelContext.get("userId")))) {
                        Tio.close(channelContext, "小黑屋用户");
                    }
                }

                // 绑定直播间群组
                Tio.bindGroup(channelContext, rid);
                // 绑定直播频道
                Tio.bindGroup(channelContext, GroupChannel.CHANNEL_LIVE);

                // 单聊视频通话cn=%@
            } else if (ChatMode.SINGLE_CHAT == mode) {
                // 单聊模式
                String cn = httpRequest.getParam(HandshakeParam.CHANNEL_NAME);
                // 绑定聊天群组
                Tio.bindGroup(channelContext, cn);
                // 绑定群组聊天频道
                Tio.bindGroup(channelContext, GroupChannel.CHANNEL_GROUP);

                // 语音房间rri=%@
            } else if (ChatMode.RTC == mode) {
                // RTC 模式
                boolean isGuest = BoolIntVal.True == Convert.toInt(channelContext.get(ImConstants.GUEST), BoolIntVal.True);
                String rri = httpRequest.getParam(HandshakeParam.RTC_ROOM_ID);
                // 直播模式
                if (!isGuest) {
                    // 判断是否被拉黑
                    if (rtcRoomService.isBlackListUser(Convert.toLong(rri), Convert.toLong(channelContext.get("userId")))) {
                        Tio.close(channelContext, "小黑屋用户");
                    }
                }

                // 绑定直播间群组
                Tio.bindGroup(channelContext, rri);
                // 绑定 RTC 频道
                Tio.bindGroup(channelContext, GroupChannel.CHANNEL_RTC);

                // 要进入群组相关时，连接时url中加rid、cn、rri这些参数
                // String rid = httpRequest.getParam(HandshakeParam.ROOM_ID);
                // String cn = httpRequest.getParam(HandshakeParam.CHANNEL_NAME);
                // String rri = httpRequest.getParam(HandshakeParam.RTC_ROOM_ID);
                // rid、cn、rri群聊相关的都没有时且uid有值就是1v1私聊模式
                // handshake方法中已设置channelContext.set(ImConstants.MODE, ChatMode.PRIVATE_CHAT);
            }else if(ChatMode.PRIVATE_CHAT == mode){
                log.info("为1v1私聊onAfterHandled方法中就和im服务端连接着，不需要加入群组---");

                // 去数据库中查找当前连接用户离线的消息，然后tio.senduser把所有离线消息发给当前连接用户（只找出未发送state=0的数据）
                PageList<PrivateMsgDO> pageList = imService.getPrivateMsgList(Convert.toLong(uid), 0, 0, 1, 100);
                log.info("数据库查找当前连接用户的消息==="+uid+"=count="+pageList.getList().size());

                for (PrivateMsgDO privateMsgDO : pageList.getList()) {
                    Tio.sendToUser(channelContext.getTioConfig(),Convert.toStr(privateMsgDO.getTargetId()) ,MsgUtil.buildMsg(privateMsgDO.getMsgData()));
                    imService.setMsgState(privateMsgDO.getId(),1);
                }
            }

        }
    }

    /**
     * 收到字节消息
     *
     * @param wsRequest
     * @param bytes
     * @param channelContext
     * @return
     * @throws Exception
     */
    @Override
    public Object onBytes(WsRequest wsRequest, byte[] bytes, ChannelContext channelContext) throws Exception {
        String msg = GzipUtil.uncompressToString(bytes);
        if (StringUtils.isNotBlank(msg)) {
            log.info("im服务端收到客户端的原始消息数据[{}] {}\n", Optional.ofNullable(channelContext.getBsId()).orElse(GUEST_TOKEN), msg);
            // 过滤心跳
            Properties properties = imServerConfig.getProps();
            if (!properties.getProperty(PropertyKeyConst.HEARTBEAT_PING).equals(msg)) {

                int mode = Convert.toInt(channelContext.get(ImConstants.MODE), 0);

                // 消息解码
                MsgPacket msgPacket = JSON.parseObject(msg, MsgPacket.class);
                log.info("im服务端收到映射msgPacket对象消息数据===\n"+MsgUtil.buildMsg(msgPacket));

                WsSessionContext wsSessionContext = (WsSessionContext) channelContext.get();

                HttpRequest httpRequest = wsSessionContext.getHandshakeRequest();

                if (mode == ChatMode.LIVE) {
                    // 游客不能说话
                    if (Convert.toInt(channelContext.get("guest"), BoolIntVal.True) == BoolIntVal.True) {
                        return null;
                    }

                    if (ImMsgType.LIVE_MSG_TEXT.equals(msgPacket.getMsg().getString("type"))) {
                        String groupId = httpRequest.getParam(HandshakeParam.ROOM_ID);
                        Long userId = Convert.toLong(channelContext.userid);
                        Long anchorUserId = Convert.toLong(groupId);

                        // 禁言判断
                        if (anchorService.isUserForbidden(Convert.toLong(groupId), userId)) {
                            msgPacket.getMsg().put("type", ImMsgType.LIVE_WORD_FORBIDDEN);
                            Tio.send(channelContext, MsgUtil.buildMsg(msgPacket));
                            return null;
                        }

                        // 发送者消息资料
                        msgPacket.setTargetType(ImTargetType.GROUP);
                        msgPacket.setTargetId(anchorUserId);
                        msgPacket.setSenderId(userId);
                        msgPacket.setSenderProfile(userService.getLiveUserProfile(userId, anchorUserId));

                        // 提及
                        long mentionUserId = msgPacket.getMsg().getLongValue("mentionUserId");
                        if (mentionUserId > 0) {
                            msgPacket.setMentionProfile(userService.getLiveUserProfile(mentionUserId, anchorUserId));
                        }

                        // 消息内容过滤
                        msgPacket.getMsg().put("text", wordFilterService.replaceMatchedPhrase(WordFilterTypeEnum.LIVE_TEXT, msgPacket.getMsg().getString("text"), "*"));

                        // 转发广播
                        QueueLiveTextMsg liveTextMsg = new QueueLiveTextMsg();
                        liveTextMsg.setGroupId(groupId);
                        liveTextMsg.setMsgPacket(msgPacket);
                        rocketProducer.sendMsg(RocketTopic.TPC_LIVE, RocketTag.LIVE_TEXT, liveTextMsg.serialize());
                    }

                } else if (mode == ChatMode.SINGLE_CHAT) {
                    // 单聊
                    if (ImMsgType.LIVE_MSG_TEXT.equals(msgPacket.getMsg().getString("type"))) {
                        String groupId = httpRequest.getParam(HandshakeParam.CHANNEL_NAME);
                        Long userId = Convert.toLong(channelContext.userid);

                        // 发送者消息资料
                        msgPacket.setTargetType(ImTargetType.GROUP);
                        msgPacket.setGroupId(groupId);
                        msgPacket.setSenderId(userId);
                        msgPacket.setSenderProfile(userService.getChatUserProfile(userId, groupId));

                        // 消息内容过滤
                        msgPacket.getMsg().put("text", wordFilterService.replaceMatchedPhrase(WordFilterTypeEnum.LIVE_TEXT, msgPacket.getMsg().getString("text"), "*"));

                        // 转发
                        // 转发广播
                        QueueLiveTextMsg liveTextMsg = new QueueLiveTextMsg();
                        liveTextMsg.setGroupId(groupId);
                        liveTextMsg.setMsgPacket(msgPacket);
                        rocketProducer.sendMsg(RocketTopic.TPC_LIVE, RocketTag.LIVE_TEXT, liveTextMsg.serialize());
                    }

                } else if (mode == ChatMode.RTC) {
                    // 游客不能说话
                    if (Convert.toInt(channelContext.get("guest"), BoolIntVal.True) == BoolIntVal.True) {
                        return null;
                    }

                    if (ImMsgType.LIVE_MSG_TEXT.equals(msgPacket.getMsg().getString("type"))) {
                        String groupId = httpRequest.getParam(HandshakeParam.RTC_ROOM_ID);
                        Long userId = Convert.toLong(channelContext.userid);
                        Long roomId = Convert.toLong(groupId);

                        // 禁言判断
                        if (rtcRoomService.isForbiddenListUser(roomId, userId)) {
                            msgPacket.getMsg().put("type", ImMsgType.LIVE_WORD_FORBIDDEN);
                            Tio.send(channelContext, MsgUtil.buildMsg(msgPacket));
                            return null;
                        }

                        // 发送者消息资料
                        msgPacket.setTargetType(ImTargetType.GROUP);
                        msgPacket.setTargetId(roomId);
                        msgPacket.setSenderId(userId);
                        msgPacket.setSenderProfile(rtcUserService.getUserProfile(userId, roomId));

                        // 提及
                        long mentionUserId = msgPacket.getMsg().getLongValue("mentionUserId");
                        if (mentionUserId > 0) {
                            msgPacket.setMentionProfile(rtcUserService.getUserProfile(mentionUserId, roomId));
                        }

                        // 消息内容过滤
                        msgPacket.getMsg().put("text", wordFilterService.replaceMatchedPhrase(WordFilterTypeEnum.LIVE_TEXT, msgPacket.getMsg().getString("text"), "*"));

                        // 转发广播
                        QueueLiveTextMsg liveTextMsg = new QueueLiveTextMsg();
                        liveTextMsg.setGroupId(groupId);
                        liveTextMsg.setMsgPacket(msgPacket);
                        rocketProducer.sendMsg(RocketTopic.TPC_LIVE, RocketTag.LIVE_TEXT, liveTextMsg.serialize());
                    } else if (ImMsgType.RTC_SEAT_VOLUME.equals(msgPacket.getMsg().getString("type"))) {
                        String groupId = httpRequest.getParam(HandshakeParam.RTC_ROOM_ID);

                        // 转发
                        Tio.sendToGroup(channelContext.getTioConfig(), groupId, MsgUtil.buildMsg(msgPacket));
                    }

                    // handshake方法中已设置channelContext.set(ImConstants.MODE, ChatMode.PRIVATE_CHAT);
                } else if (mode == ChatMode.PRIVATE_CHAT){
                    // 游客不能说话
                    if (Convert.toInt(channelContext.get("guest"), BoolIntVal.True) == BoolIntVal.True) {
                        return null;
                    }

                    // 这儿就相当于api-app层，调用service
                    String senderId = Convert.toStr(msgPacket.getSenderProfile().getUserId());
                    String targetId = Convert.toStr(msgPacket.getTargetProfile().getUserId());
                    boolean online = redisService.sIsMember(RedisUserKeys.USER_ONLINE, targetId);
                    int targetType = msgPacket.getTargetType();
                    String msgType = msgPacket.getMsg().getString("type");
                    String msgContent = msgPacket.getMsg().getString("text");

                    // 内容检测
                    if(msgContent.contains("shabi")||msgContent.contains("sb")||msgContent.contains("傻逼")){
                        // 违规
                        msgPacket.setMsgState(10);
                    }

                    if(online){
                        log.info("目标者在线，消息转发给 目标者 和 发送者 并存起来===目标\n"+targetId+msg);
                        // 消息不违规只才发给目标者
                        int msgState = msgPacket.getMsgState() == 10 ? 10 : 1;
                        if (msgPacket.getMsgState() != 10) {
                            Tio.sendToUser(channelContext.getTioConfig(), targetId, MsgUtil.buildMsg(msgPacket));
                        }
                        Tio.sendToUser(channelContext.getTioConfig(),senderId,MsgUtil.buildMsg(msgPacket));
                        imService.addPrivateMsg(msgPacket.getMsgId(), targetType,msgState, Convert.toLong(senderId), Convert.toLong(targetId), msgType, msg);

                    }else{
                        // 把要发送的消息存起来
                        log.info("目标者不在线，消息转发给 发送者 并存起来===");
                        int msgState = msgPacket.getMsgState() == 10 ? 10 : 0;
                        Tio.sendToUser(channelContext.getTioConfig(),senderId,MsgUtil.buildMsg(msgPacket));
                        imService.addPrivateMsg(msgPacket.getMsgId(), targetType,msgState, Convert.toLong(senderId), Convert.toLong(targetId), msgType, msg);
                    }
                }

            } else {
                Tio.send(channelContext, MsgUtil.buildMsg(properties.getProperty(PropertyKeyConst.HEARTBEAT_PONG)));
            }
        }

        return null;
    }

    /**
     * 连接关闭
     *
     * @param wsRequest
     * @param bytes
     * @param channelContext
     * @return
     * @throws Exception
     */
    @Override
    public Object onClose(WsRequest wsRequest, byte[] bytes, ChannelContext channelContext) throws Exception {
        Tio.close(channelContext, "client close");
        log.info("onClose了===");
        return null;
    }

    /**
     * 收到文本消息
     *
     * @param wsRequest
     * @param s
     * @param channelContext
     * @return
     * @throws Exception
     */
    @Override
    public Object onText(WsRequest wsRequest, String s, ChannelContext channelContext) throws Exception {
        log.info("收到文本消息[{}] {}", channelContext.getBsId(), s);
        return null;
    }
}
