package com.starbuds.server.im.server.ws.listener;

import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.constant.cache.RedisUserKeys;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.pojo.mq.*;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.im.server.constant.ChatMode;
import com.starbuds.server.im.server.constant.GroupChannel;
import com.starbuds.server.im.server.constant.ImConstants;
import com.starbuds.server.im.server.rocket.RocketProducer;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.tio.core.ChannelContext;
import org.tio.core.intf.GroupListener;
import org.apache.dubbo.config.annotation.Reference;

import javax.annotation.Resource;

/**
 * 备注
 *
 * @author Lay
 * @date 2020/1/5
 */
@Component
@Slf4j
public class ImGroupListener implements GroupListener {

    @Resource
    RocketProducer rocketProducer;

    @Reference(version = "1.0.0")
    RedisService redisService;

    /**
     * 绑定群组后
     *
     * @param channelContext
     * @param group
     * @throws Exception
     */
    @Override
    public void onAfterBind(ChannelContext channelContext, String group) throws Exception {
        // 过滤频道
        if (GroupChannel.CHANNEL_LIVE.equals(group) || GroupChannel.CHANNEL_GROUP.equals(group) || GroupChannel.CHANNEL_RTC.equals(group)) {
            return;
        }
        int mode = Convert.toInt(channelContext.get(ImConstants.MODE), 0);
        if (ChatMode.LIVE == mode) {
            if (Convert.toInt(channelContext.get(ImConstants.GUEST), BoolIntVal.True) == BoolIntVal.False) {
                String bsId = channelContext.getBsId();
                bsId = bsId.split("_")[1];
                // 发送加入直播间消息
                QueueLiveUserJoinMsg liveUserJoinMsg = new QueueLiveUserJoinMsg();
                liveUserJoinMsg.setJoinUserId(Convert.toLong(bsId));
                liveUserJoinMsg.setAnchorUserId(Convert.toLong(group));
                liveUserJoinMsg.setChannelId(Convert.toLong(channelContext.get("channelId"), 0L));
                rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.LIVE_USER_JOIN, liveUserJoinMsg.serialize());
            }

        } else if (ChatMode.SINGLE_CHAT == mode) {
            String bsId = channelContext.getBsId();
            bsId = bsId.split("_")[1];
            // 单聊模式
            QueueSingleChatUserJoinMsg singleChatUserJoinMsg = new QueueSingleChatUserJoinMsg();
            singleChatUserJoinMsg.setJoinUserId(Convert.toLong(bsId));
            singleChatUserJoinMsg.setChannelName(group);
            singleChatUserJoinMsg.setChannelId(Convert.toLong(channelContext.get("channelId"), 0L));
            rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.SINGLE_CHAT_USER_JOIN, singleChatUserJoinMsg.serialize());

        } else if (ChatMode.RTC == mode) {
            // RTC 模式
            if (Convert.toInt(channelContext.get(ImConstants.GUEST), BoolIntVal.True) == BoolIntVal.False) {
                String bsId = channelContext.getBsId();
                bsId = bsId.split("_")[1];
                // 发送进入RTC房间消息
                QueueRtcUserJoinMsg rtcUserJoinMsg = new QueueRtcUserJoinMsg();
                rtcUserJoinMsg.setJoinUserId(Convert.toLong(bsId));
                rtcUserJoinMsg.setRoomId(Convert.toLong(group));
                rtcUserJoinMsg.setChannelId(Convert.toLong(channelContext.get("channelId"), 0L));

                String appVer = redisService.get(RedisUserKeys.getUserAppVer(Convert.toLong(bsId)));
                if (appVer == null) {
                    appVer = "1.8.0";
                }
                rtcUserJoinMsg.setAppVer(appVer);
                rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.RTC_USER_JOIN, rtcUserJoinMsg.serialize());
            }
        }
    }

    /**
     * 解绑群组后
     *
     * @param channelContext
     * @param group
     * @throws Exception
     */
    @Override
    public void onAfterUnbind(ChannelContext channelContext, String group) throws Exception {
        // 过滤频道
        if (GroupChannel.CHANNEL_LIVE.equals(group) || GroupChannel.CHANNEL_GROUP.equals(group) || GroupChannel.CHANNEL_RTC.equals(group)) {
            return;
        }
        int mode = Convert.toInt(channelContext.get(ImConstants.MODE), 0);
        if (ChatMode.LIVE == mode) {
            // 直播模式
            if (Convert.toInt(channelContext.get(ImConstants.GUEST), BoolIntVal.True) == BoolIntVal.False) {
                String bsId = channelContext.getBsId();
                bsId = bsId.split("_")[1];
                // 发送离开直播间消息
                QueueLiveUserExitMsg liveUserExitMsg = new QueueLiveUserExitMsg();
                liveUserExitMsg.setExitUserId(Convert.toLong(bsId));
                liveUserExitMsg.setAnchorUserId(Convert.toLong(group));
                liveUserExitMsg.setChannelId(Convert.toLong(channelContext.get("channelId"), 0L));
                rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.LIVE_USER_EXIT, liveUserExitMsg.serialize());
            }

        } else if (ChatMode.SINGLE_CHAT == mode) {
            String bsId = channelContext.getBsId();
            bsId = bsId.split("_")[1];
            // 单聊模式
            QueueSingleChatUserExitMsg singleChatUserExitMsg = new QueueSingleChatUserExitMsg();
            singleChatUserExitMsg.setExitUserId(Convert.toLong(bsId));
            singleChatUserExitMsg.setChannelName(group);
            singleChatUserExitMsg.setChannelId(Convert.toLong(channelContext.get("channelId"), 0L));
            rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.SINGLE_CHAT_USER_EXIT, singleChatUserExitMsg.serialize());

        } else if (ChatMode.RTC == mode) {
            // RTC 模式
            if (Convert.toInt(channelContext.get(ImConstants.GUEST), BoolIntVal.True) == BoolIntVal.False) {
                String bsId = channelContext.getBsId();
                bsId = bsId.split("_")[1];
                // 发送离开直播间消息
                QueueRtcUserExitMsg rtcUserExitMsg = new QueueRtcUserExitMsg();
                rtcUserExitMsg.setExitUserId(Convert.toLong(bsId));
                rtcUserExitMsg.setRoomId(Convert.toLong(group));
                rtcUserExitMsg.setChannelId(Convert.toLong(channelContext.get("channelId"), 0L));
                rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.RTC_USER_EXIT, rtcUserExitMsg.serialize());
            }
        }
    }
}
