package com.starbuds.server.im.server.ws.listener;

import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.constant.cache.RedisUserKeys;
import com.starbuds.server.common.service.data.RedisService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;
import org.tio.core.ChannelContext;
import org.tio.core.Tio;
import org.tio.core.intf.Packet;
import org.tio.websocket.common.WsResponse;
import org.tio.websocket.common.WsSessionContext;
import org.tio.websocket.server.WsServerAioListener;

/**
 * 备注
 *    NSString *url = [NSString stringWithFormat:@"%@?token=%@&%@&cid=%d", ApplicationDelegate.appConfigModel.channelConfig.liveMsgServer, [self URLEncodedString:[DataBase nowUser].token], groupId,APP_Channel_Id];
 *     [self.socketManager connectServerWithUrl:url connectCompletionHandler:^(int errCode, NSString * _Nonnull errMsg) {
 *         if (errCode == 0) {
 *             // 保存当前直播间会话id
 *             self.groupId = groupId;
 *
 *             // 直播间提示
 *             [self greenMessageTips];
 *             !succ?:succ();
 *         }else {
 *             !fail?:fail(errCode, errMsg);
 *         }
 *         //NSLog(@"建立长链接错误信息:%@",errMsg);
 *     }];
 * @author Lay
 * @date 2020/1/5
 */
@Slf4j
@Component
public class ImServerAioListener extends WsServerAioListener {

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Override
    public void onAfterConnected(ChannelContext channelContext, boolean isConnected, boolean isReconnect) throws Exception {
        super.onAfterConnected(channelContext, isConnected, isReconnect);
        if (log.isInfoEnabled()) {
            // log.info("onAfterConnected\r\n{}", channelContext);
        }
    }

    @Override
    public void onAfterHandled(ChannelContext channelContext, Packet packet, long cost) throws Exception {
        super.onAfterHandled(channelContext, packet, cost);
        if (log.isInfoEnabled()) {
            // log.info("onAfterHandled\r\n{}\r\n{}", packet.logstr(), channelContext);
        }
    }

    @Override
    public void onBeforeClose(ChannelContext channelContext, Throwable throwable, String remark, boolean isRemove) throws Exception {
        super.onBeforeClose(channelContext, throwable, remark, isRemove);
        if (log.isInfoEnabled()) {
            // log.info("onBeforeClose\r\n{}", channelContext);
        }
        long userId = Convert.toLong(channelContext.get("userId"));
        log.info("连接关闭==="+userId);
        if(userId > 0){
            // 移除用户在线状态
            redisService.sRemove(RedisUserKeys.USER_ONLINE, Convert.toStr(userId));
        }

        WsSessionContext wsSessionContext = (WsSessionContext) channelContext.get();

        if (wsSessionContext != null && wsSessionContext.isHandshaked()) {

            // int count = Tio.getAll(channelContext.tioConfig).getObj().size();
            //
            // String msg = channelContext.getClientNode().toString() + " 离开了，现在共有【" + count + "】人在线";
            // //用tio-websocket，服务器发送到客户端的Packet都是WsResponse
            // WsResponse wsResponse = WsResponse.fromText(msg, ShowcaseServerConfig.CHARSET);
            // //群发
            // Tio.sendToGroup(channelContext.tioConfig, Const.GROUP_ID, wsResponse);
        }
    }

    @Override
    public void onAfterSent(ChannelContext channelContext, Packet packet, boolean isSentSuccess) throws Exception {
        super.onAfterSent(channelContext, packet, isSentSuccess);
        if (log.isInfoEnabled()) {
            // log.info("onAfterSent\r\n{}\r\n{}", packet.logstr(), channelContext);
        }
    }

    @Override
    public void onAfterDecoded(ChannelContext channelContext, Packet packet, int packetSize) throws Exception {
        super.onAfterDecoded(channelContext, packet, packetSize);
        if (log.isInfoEnabled()) {
            // log.info("onAfterDecoded\r\n{}\r\n{}", packet.logstr(), channelContext);
        }
    }

    @Override
    public void onAfterReceivedBytes(ChannelContext channelContext, int receivedBytes) throws Exception {
        super.onAfterReceivedBytes(channelContext, receivedBytes);
        if (log.isInfoEnabled()) {
            // log.info("onAfterReceivedBytes\r\n{}", channelContext);
        }
    }

}
